/***************************************************************************                          
    file:           generalcfgdesign.cpp
    description:    general configuration design
    begin:          09/06/2024
    Copyright:      Giuseppe Della Bianca<giusdbg@gmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qvariant.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qgroupbox.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qapplication.h>

#include <kaboutdata.h>
#include <klocalizedstring.h>

#include "general.h"
#include "generalcfgdesign.h"


/*
 *  Constructs a GeneralCfgDesign as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 */
GeneralCfgDesign::GeneralCfgDesign( QWidget* parent, const QString &name, Qt::WindowFlags fl )
    : QWidget( parent /*, name */, fl )
{
    QGridLayout *qglAppear, *qglPref, *qglUpdTime;
    QVBoxLayout *qvblGaugeSize, *qvblChartSize;
    QHBoxLayout *qhblChartSize;
    
    if ( name.isEmpty() )
	setObjectName( "GeneralCfgDesign" );

    GeneralCfgDesignLayout = new QGridLayout( this ); 
    
    qtwMain = new QTabWidget( this );
    
    qwAppear = new QWidget( qtwMain );
    qglAppear = new QGridLayout( qwAppear );
    
    qwPref = new QWidget( qtwMain );
    qglPref = new QGridLayout( qwPref );
        
    qgbMeterAppear = new QGroupBox( qwAppear /*, "qgbMeterAppear" */ );
    qgbMeterAppear->setAlignment( Qt::Vertical );
//    qgbMeterAppear->layout()->setSpacing( 6 );
//    qgbMeterAppear->layout()->setMargin( 11 ); 
    qglMeterAppear = new QGridLayout( qgbMeterAppear );
    qglMeterAppear->setAlignment( Qt::AlignTop );


    qvblGaugeSize = new QVBoxLayout();
    qlGaugeSize = new QLabel( qgbMeterAppear );
    qcbGaugeSize = new QComboBox( qgbMeterAppear );
//    qcbGaugeSize->setGeometry( QRect(160, 50, 87, 32) );

    qvblChartSize = new QVBoxLayout();
    qhblChartSize = new QHBoxLayout();
    qlChartSize = new QLabel( qgbMeterAppear );
    qlChartSizeSep = new QLabel( qgbMeterAppear );
    qlChartSizeSep->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred );
    qcbChartWidth = new QComboBox( qgbMeterAppear );
    qcbChartHeight = new QComboBox( qgbMeterAppear );

    addMeterSize();
    
    qvblGaugeSize->addWidget( qlGaugeSize );
    qvblGaugeSize->addWidget( qcbGaugeSize );

    qvblChartSize->addWidget( qlChartSize );
    qvblChartSize->addLayout( qhblChartSize );
    qhblChartSize->addWidget( qcbChartWidth );
    qhblChartSize->addWidget( qlChartSizeSep );
    qhblChartSize->addWidget( qcbChartHeight );

    qglMeterAppear->addLayout( qvblGaugeSize, 0, 0, 1, 1 );
    qglMeterAppear->addLayout( qvblChartSize, 0, 1, 1, 1 );

//    qglMeterAppear->addWidget ( qcbPanelSize, 0, 0 );    
    qglAppear->addWidget( qgbMeterAppear, 0, 0 );

    qcbPanelSolid = new QCheckBox ( qwAppear );
    qcbPanelSolid->setChecked( true );
    
    qglMeterAppear->addWidget( qcbPanelSolid, 1, 0 );
    
    qgbPanelOpacity = new QGroupBox( qwAppear );
    qgbPanelOpacity->setFlat( true );
    qhblPanelOpacity = new QHBoxLayout( qgbPanelOpacity );
    qhblPanelOpacity->setAlignment( Qt::AlignLeft );
    qhblPanelOpacity->setContentsMargins( 0, 0, 0, 0 );
    
    qlPanelOpacityLabel = new QLabel ( qwAppear );
    qlPanelOpacityLabel->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred );
    qdsbPanelOpacityValue = new QDoubleSpinBox ( qwAppear );
    qdsbPanelOpacityValue->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred );
    qdsbPanelOpacityValue->setDecimals ( 0 );
    qdsbPanelOpacityValue->setMinimum( 0.0 );
    qdsbPanelOpacityValue->setMaximum( 100.0 );
    qdsbPanelOpacityValue->setValue( 70 );
    qsiPanelOpacitySpacer = new QSpacerItem( 0, 0, QSizePolicy::Maximum, QSizePolicy::Preferred );
    
    qhblPanelOpacity->addWidget ( qlPanelOpacityLabel );
    qhblPanelOpacity->addWidget ( qdsbPanelOpacityValue );    
    qhblPanelOpacity->addItem( qsiPanelOpacitySpacer );
    qglMeterAppear->addWidget( qgbPanelOpacity, 1, 1 );

    qcbWindowsSolid = new QCheckBox ( qwAppear );
    qcbWindowsSolid->setChecked( true );

    qglMeterAppear->addWidget( qcbWindowsSolid, 2, 0 );
    
    qgbWindowsOpacity = new QGroupBox( qwAppear );
    qgbWindowsOpacity->setFlat( true );
    qhblWindowsOpacity = new QHBoxLayout( qgbWindowsOpacity );
    qhblWindowsOpacity->setContentsMargins( 0, 0, 0, 0 );
    
    qlWindowsOpacityLabel = new QLabel ( qwAppear );
    qlWindowsOpacityLabel->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred );
    qdsbWindowsOpacityValue = new QDoubleSpinBox ( qwAppear );
    qdsbWindowsOpacityValue->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred );
    qdsbWindowsOpacityValue->setDecimals ( 0 );
    qdsbWindowsOpacityValue->setMinimum( 0.0 );
    qdsbWindowsOpacityValue->setMaximum( 100.0 );
    qdsbWindowsOpacityValue->setValue( 70 );
    qsiWindowsOpacitySpacer = new QSpacerItem( 0, 0, QSizePolicy::Maximum, QSizePolicy::Preferred );
    
    qhblWindowsOpacity->addWidget ( qlWindowsOpacityLabel );
    qhblWindowsOpacity->addWidget ( qdsbWindowsOpacityValue );    
    qhblWindowsOpacity->addItem( qsiWindowsOpacitySpacer );
    qglMeterAppear->addWidget( qgbWindowsOpacity, 2, 1 );
    
    boxGeneralPalette = new QGroupBox( qwAppear );
    boxGeneralPalette->setMinimumSize( QSize( 0, 90 ) );
    qglAppear->addWidget( boxGeneralPalette, 1, 0 );
    
    GroupBox2 = new QGroupBox( qwAppear );
    GroupBox2->setAlignment( Qt::Vertical );
//    GroupBox2->layout()->setSpacing( 6 );
//    GroupBox2->layout()->setMargin( 11 );
    GroupBox2Layout = new QVBoxLayout( GroupBox2 );
    GroupBox2Layout->setAlignment( Qt::AlignTop );

    CheckBoxAutoStart = new QCheckBox( GroupBox2 /*, "CheckBoxAutoStart" */ );
    GroupBox2Layout->addWidget( CheckBoxAutoStart );

    qglAppear->addWidget( GroupBox2, 2, 0 );
    Spacer28 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );        
    qglAppear->addItem( Spacer28, 3, 0 );
    
    qtwMain->addTab( qwAppear, QString() );
        
    qgbUpdTime = new QGroupBox( qwPref );
    qgbUpdTime->setAlignment( Qt::Vertical );
    
    qglUpdTime = new QGridLayout( qgbUpdTime );
    qglUpdTime->setAlignment( Qt::AlignTop );

    qdsbUpdTime = new QDoubleSpinBox( qgbUpdTime );
    qdsbUpdTime->setSuffix ( qsFL1( "s" ) );
    qdsbUpdTime->setDecimals( 1 );
    qdsbUpdTime->setSingleStep( 0.1 );
    qdsbUpdTime->setMinimum( 0.5 );
    qdsbUpdTime->setValue( 5.0 );

    qglUpdTime->addWidget( qdsbUpdTime, 0, 0 );

    qsUpdTime = new QSlider( qgbUpdTime );
    qsUpdTime->setPageStep( 5 );
    qsUpdTime->setSingleStep( 5 );
    qsUpdTime->setMinimum( 5 );
    qsUpdTime->setMaximum( 600 );
    qsUpdTime->setValue( 50 );
    qsUpdTime->setOrientation( Qt::Horizontal );

    qglUpdTime->addWidget( qsUpdTime, 0, 1 );

    qglPref->addWidget( qgbUpdTime, 1, 0 );
    qtwMain->addTab( qwPref, QString() );

    GeneralCfgDesignLayout->addWidget( qtwMain, 0, 0 );
    
    languageChange();
//    resize( QSize(343, 239).expandedTo(minimumSizeHint()) );
//   clearWState( WState_Polished );
    
    connect( qcbPanelSolid, &QAbstractButton::toggled, this, &GeneralCfgDesign::slotPanelSolidToggled );
    connect( qcbWindowsSolid, &QAbstractButton::toggled, this, &GeneralCfgDesign::slotWindowsSolidToggled );    
    connect( qsUpdTime, SIGNAL( valueChanged( int) ), this, SLOT( slotUpdateTimeSliderValue( int ) ) );
    connect( qdsbUpdTime, SIGNAL( valueChanged( double ) ), this, SLOT( slotUpdateTimeSpinValue( double ) ) );
}

/*
 *  Destroys the object and frees any allocated resources
 */
GeneralCfgDesign::~GeneralCfgDesign()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void GeneralCfgDesign::languageChange()
{
    const KAboutData kadAboutData = KAboutData::applicationData();
    
    setWindowTitle( tr2i18n( "GeneralCfg" ) );
        
    qgbMeterAppear->setTitle( tr2i18n( "Panels appearance" ) );

    qlGaugeSize->setText ( qsFL1( "Gauges size" ) );
    qcbGaugeSize->setCurrentIndex( qcbGaugeSize->findData ( qsFL1( "64 x 64" ), ( Qt::UserRole + 1 ) ) );
    
    qlChartSize->setText ( qsFL1( "Chart size" ) );
    qlChartSizeSep->setText ( qsFL1( " x " ) );
    qcbChartWidth->setCurrentIndex( qcbChartWidth->findData ( qsFL1( "96" ), ( Qt::UserRole + 1 ) ) );
    qcbChartHeight->setCurrentIndex ( qcbChartHeight->findData ( qsFL1( "96" ), ( Qt::UserRole + 1 ) ) );

    qcbPanelSolid->setText( qsFL1( "Solid" ) );
    qlPanelOpacityLabel->setText( qsFL1( "Opacity" ) );
    qdsbPanelOpacityValue->setSuffix( qsFL1( "%" ) );

    qcbWindowsSolid->setText( qsFL1( "Windows Solid" ) );
    qlWindowsOpacityLabel->setText( qsFL1( "Windows Opacity" ) );
    qdsbWindowsOpacityValue->setSuffix( qsFL1( "%" ) );


    boxGeneralPalette->setTitle( tr2i18n( "Colors" ) );
    GroupBox2->setTitle( tr2i18n( "Startup" ) );
    CheckBoxAutoStart->setText( tr2i18n( ( qsFL1( "Autostart " ) + kadAboutData.displayName() + qsFL1( " on KDE startup" ) ).toLatin1().data() ) );
    
    qtwMain->setTabText( qtwMain->indexOf( qwAppear ), tr2i18n( "Appearance" ) );
    qtwMain->setTabText( qtwMain->indexOf( qwPref ), tr2i18n( "Preferences" ) );
    
    qgbUpdTime->setTitle( tr2i18n( "Update interval" ) );

}

/***************************************************************************
    add meter size
 ***************************************************************************/

void GeneralCfgDesign::addMeterSize()
{
    qcbGaugeSize->insertItem( 0, tr2i18n( "160 x 160" ) );
    qcbGaugeSize->insertItem( 1, tr2i18n( "152 x 152" ) );
    qcbGaugeSize->insertItem( 2, tr2i18n( "144 x 144" ) );
    qcbGaugeSize->insertItem( 3, tr2i18n( "136 x 136" ) );
    qcbGaugeSize->insertItem( 4, tr2i18n( "128 x 128" ) );
    qcbGaugeSize->insertItem( 5, tr2i18n( "120 x 120" ) );
    qcbGaugeSize->insertItem( 6, tr2i18n( "112 x 112" ) );
    qcbGaugeSize->insertItem( 7, tr2i18n( "104 x 104" ) );
    qcbGaugeSize->insertItem( 8, tr2i18n( "96 x 96" ) );
    qcbGaugeSize->insertItem( 9, tr2i18n( "88 x 88" ) );
    qcbGaugeSize->insertItem( 10, tr2i18n( "80 x 80" ) );
    qcbGaugeSize->insertItem( 11, tr2i18n( "72 x 72" ) );
    qcbGaugeSize->insertItem( 12, tr2i18n( "64 x 64" ) );
    qcbGaugeSize->insertItem( 13, tr2i18n( "56 x 56" ) );
    qcbGaugeSize->insertItem( 14, tr2i18n( "48 x 48" ) );

    qcbGaugeSize->setItemData( 0, qsFL1( "160" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 1, qsFL1( "152" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 2, qsFL1( "144" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 3, qsFL1( "136" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 4, qsFL1( "128" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 5, qsFL1( "120" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 6, qsFL1( "112" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 7, qsFL1( "104" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 8, qsFL1( "96" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 9, qsFL1( "88" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 10, qsFL1( "80" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 11, qsFL1( "72" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 12, qsFL1( "64" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 13, qsFL1( "56" ), ( Qt::UserRole + 1 ) );
    qcbGaugeSize->setItemData( 14, qsFL1( "48" ), ( Qt::UserRole + 1 ) );


    qcbChartWidth->insertItem( 0, tr2i18n( "960" ) );
    qcbChartWidth->insertItem( 1, tr2i18n( "864" ) );
    qcbChartWidth->insertItem( 2, tr2i18n( "768" ) );
    qcbChartWidth->insertItem( 3, tr2i18n( "672" ) );
    qcbChartWidth->insertItem( 4, tr2i18n( "576" ) );
    qcbChartWidth->insertItem( 5, tr2i18n( "480" ) );
    qcbChartWidth->insertItem( 6, tr2i18n( "384" ) );
    qcbChartWidth->insertItem( 7, tr2i18n( "288" ) );
    qcbChartWidth->insertItem( 8, tr2i18n( "192" ) );
    qcbChartWidth->insertItem( 9, tr2i18n( "160" ) );
    qcbChartWidth->insertItem( 10, tr2i18n( "144" ) );
    qcbChartWidth->insertItem( 11, tr2i18n( "128" ) );
    qcbChartWidth->insertItem( 12, tr2i18n( "112" ) );
    qcbChartWidth->insertItem( 13, tr2i18n( "96" ) );

    qcbChartWidth->setItemData( 0, qsFL1( "960" ), ( Qt::UserRole + 1 ) );
    qcbChartWidth->setItemData( 1, qsFL1( "864" ), ( Qt::UserRole + 1 ) );
    qcbChartWidth->setItemData( 2, qsFL1( "768" ), ( Qt::UserRole + 1 ) );
    qcbChartWidth->setItemData( 3, qsFL1( "672" ), ( Qt::UserRole + 1 ) );
    qcbChartWidth->setItemData( 4, qsFL1( "576" ), ( Qt::UserRole + 1 ) );
    qcbChartWidth->setItemData( 5, qsFL1( "480" ), ( Qt::UserRole + 1 ) );
    qcbChartWidth->setItemData( 6, qsFL1( "384" ), ( Qt::UserRole + 1 ) );
    qcbChartWidth->setItemData( 7, qsFL1( "288" ), ( Qt::UserRole + 1 ) );
    qcbChartWidth->setItemData( 8, qsFL1( "192" ), ( Qt::UserRole + 1 ) );
    qcbChartWidth->setItemData( 9, qsFL1( "160" ), ( Qt::UserRole + 1 ) );
    qcbChartWidth->setItemData( 10, qsFL1( "144" ), ( Qt::UserRole + 1 ) );
    qcbChartWidth->setItemData( 11, qsFL1( "128" ), ( Qt::UserRole + 1 ) );
    qcbChartWidth->setItemData( 12, qsFL1( "112" ), ( Qt::UserRole + 1 ) );
    qcbChartWidth->setItemData( 13, qsFL1( "96" ), ( Qt::UserRole + 1 ) );

    qcbChartHeight->insertItem( 0, tr2i18n( "480" ) );
    qcbChartHeight->insertItem( 1, tr2i18n( "384" ) );
    qcbChartHeight->insertItem( 2, tr2i18n( "288" ) );
    qcbChartHeight->insertItem( 3, tr2i18n( "192" ) );
    qcbChartHeight->insertItem( 5, tr2i18n( "160" ) );
    qcbChartHeight->insertItem( 5, tr2i18n( "144" ) );
    qcbChartHeight->insertItem( 6, tr2i18n( "128" ) );
    qcbChartHeight->insertItem( 7, tr2i18n( "112" ) );
    qcbChartHeight->insertItem( 8, tr2i18n( "96" ) );

    qcbChartHeight->setItemData( 0, qsFL1( "480" ), ( Qt::UserRole + 1 ) );
    qcbChartHeight->setItemData( 1, qsFL1( "384" ), ( Qt::UserRole + 1 ) );
    qcbChartHeight->setItemData( 2, qsFL1( "288" ), ( Qt::UserRole + 1 ) );
    qcbChartHeight->setItemData( 3, qsFL1( "192" ), ( Qt::UserRole + 1 ) );
    qcbChartHeight->setItemData( 4, qsFL1( "160" ), ( Qt::UserRole + 1 ) );
    qcbChartHeight->setItemData( 5, qsFL1( "144" ), ( Qt::UserRole + 1 ) );
    qcbChartHeight->setItemData( 6, qsFL1( "128" ), ( Qt::UserRole + 1 ) );
    qcbChartHeight->setItemData( 7, qsFL1( "112" ), ( Qt::UserRole + 1 ) );
    qcbChartHeight->setItemData( 8, qsFL1 ( "96" ), ( Qt::UserRole + 1 ) );

} // void addMeterSize()

void GeneralCfgDesign::slotPanelSolidToggled( bool checked )
{
    if ( checked )
        qdsbPanelOpacityValue->setValue( 100.0 );
    else
        qdsbPanelOpacityValue->setValue( 70.0 );
}

void GeneralCfgDesign::slotWindowsSolidToggled( bool checked )
{
    if ( checked )
        qdsbWindowsOpacityValue->setValue( 100.0 );
    else
        qdsbWindowsOpacityValue->setValue( 70.0 );
}

void GeneralCfgDesign::slotUpdateTimeSliderValue( int value )
{
    qdsbUpdTime->setValue( ( value / 10.0 ) );        
}

void GeneralCfgDesign::slotUpdateTimeSpinValue( double value )
{
    qsUpdTime->setValue( ( value * 10.0 ) );        
}

//#include "generalcfgdesign.moc"
